package sun.font;

import java.awt.Rectangle;
import java.awt.GraphicsEnvironment;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.GeneralPath;
import java.lang.ref.WeakReference;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.lang.ref.SoftReference;
import java.awt.geom.Rectangle2D;
import java.util.concurrent.ConcurrentHashMap;

public class FileFontStrike extends PhysicalStrike
{
    static final int INVISIBLE_GLYPHS = 65534;
    private FileFont fileFont;
    private static final int UNINITIALISED = 0;
    private static final int INTARRAY = 1;
    private static final int LONGARRAY = 2;
    private static final int SEGINTARRAY = 3;
    private static final int SEGLONGARRAY = 4;
    private volatile int glyphCacheFormat;
    private static final int SEGSHIFT = 5;
    private static final int SEGSIZE = 32;
    private boolean segmentedCache;
    private int[][] segIntGlyphImages;
    private long[][] segLongGlyphImages;
    private float[] horizontalAdvances;
    private float[][] segHorizontalAdvances;
    ConcurrentHashMap<Integer, Rectangle2D.Float> boundsMap;
    SoftReference<ConcurrentHashMap<Integer, Point2D.Float>> glyphMetricsMapRef;
    AffineTransform invertDevTx;
    boolean useNatives;
    NativeStrike[] nativeStrikes;
    private int intPtSize;
    private static boolean isXPorLater;
    private WeakReference<ConcurrentHashMap<Integer, GeneralPath>> outlineMapRef;
    
    private static native boolean initNative();
    
    FileFontStrike(FileFont fileFont, FontStrikeDesc fontStrikeDesc) {
        super(fileFont, fontStrikeDesc);
        this.glyphCacheFormat = 0;
        this.fileFont = fileFont;
        if (fontStrikeDesc.style != fileFont.style) {
            if ((fontStrikeDesc.style & 0x2) == 0x2 && (fileFont.style & 0x2) == 0x0) {
                this.algoStyle = true;
                this.italic = 0.7f;
            }
            if ((fontStrikeDesc.style & 0x1) == 0x1 && (fileFont.style & 0x1) == 0x0) {
                this.algoStyle = true;
                this.boldness = 1.33f;
            }
        }
        double[] flatmatrix = new double[4];
        AffineTransform glyphTx = fontStrikeDesc.glyphTx;
        glyphTx.getMatrix(flatmatrix);
        if (!fontStrikeDesc.devTx.isIdentity() && fontStrikeDesc.devTx.getType() != 1) {
            try {
                this.invertDevTx = fontStrikeDesc.devTx.createInverse();
            }
            catch (NoninvertibleTransformException ex) {}
        }
        int n = fontStrikeDesc.aaHint != 1 && fileFont.familyName.startsWith("Amble");
        if (Double.isNaN(flatmatrix[0]) || Double.isNaN(flatmatrix[1]) || Double.isNaN(flatmatrix[2]) || Double.isNaN(flatmatrix[3]) || fileFont.getScaler() == null) {
            this.pScalerContext = NullFontScaler.getNullScalerContext();
        }
        else {
            this.pScalerContext = fileFont.getScaler().createScalerContext(flatmatrix, fontStrikeDesc.aaHint, fontStrikeDesc.fmHint, this.boldness, this.italic, (boolean)n);
        }
        this.mapper = fileFont.getMapper();
        int numGlyphs = this.mapper.getNumGlyphs();
        float n2 = (float)flatmatrix[3];
        int intPtSize = (int)n2;
        this.intPtSize = intPtSize;
        int n3 = intPtSize;
        int n4 = (glyphTx.getType() & 0x7C) == 0x0;
        this.segmentedCache = (numGlyphs > 256 || (numGlyphs > 64 && (n4 == 0 || n2 != (float)n3 || n3 < 6 || n3 > 36)));
        if (this.pScalerContext == 0L) {
            this.disposer = new FontStrikeDisposer(fileFont, fontStrikeDesc);
            this.initGlyphCache();
            this.pScalerContext = NullFontScaler.getNullScalerContext();
            SunFontManager.getInstance().deRegisterBadFont(fileFont);
            return;
        }
        if (FontUtilities.isWindows && FileFontStrike.isXPorLater && !FontUtilities.useT2K && !GraphicsEnvironment.isHeadless() && !fileFont.useJavaRasterizer && (fontStrikeDesc.aaHint == 4 || fontStrikeDesc.aaHint == 5) && flatmatrix[1] == 0.0 && flatmatrix[2] == 0.0 && flatmatrix[0] == flatmatrix[3] && flatmatrix[0] >= 3.0 && flatmatrix[0] <= 100.0 && !((TrueTypeFont)fileFont).useEmbeddedBitmapsForSize(this.intPtSize)) {
            this.useNatives = true;
        }
        else if (fileFont.checkUseNatives() && fontStrikeDesc.aaHint == 0 && !this.algoStyle && flatmatrix[1] == 0.0 && flatmatrix[2] == 0.0 && flatmatrix[0] >= 6.0 && flatmatrix[0] <= 36.0 && flatmatrix[0] == flatmatrix[3]) {
            this.useNatives = true;
            int length = fileFont.nativeFonts.length;
            this.nativeStrikes = new NativeStrike[length];
            for (int i = 0; i < length; ++i) {
                this.nativeStrikes[i] = new NativeStrike(fileFont.nativeFonts[i], fontStrikeDesc, false);
            }
        }
        if (FontUtilities.isLogging() && FontUtilities.isWindows) {
            FontUtilities.getLogger().info("Strike for " + fileFont + " at size = " + this.intPtSize + " use natives = " + this.useNatives + " useJavaRasteriser = " + fileFont.useJavaRasterizer + " AAHint = " + fontStrikeDesc.aaHint + " Has Embedded bitmaps = " + ((TrueTypeFont)fileFont).useEmbeddedBitmapsForSize(this.intPtSize));
        }
        this.disposer = new FontStrikeDisposer(fileFont, fontStrikeDesc, this.pScalerContext);
        double n5 = 48.0;
        if (!(this.getImageWithAdvance = (Math.abs(glyphTx.getScaleX()) <= n5 && Math.abs(glyphTx.getScaleY()) <= n5 && Math.abs(glyphTx.getShearX()) <= n5 && Math.abs(glyphTx.getShearY()) <= n5))) {
            if (!this.segmentedCache) {
                this.horizontalAdvances = new float[numGlyphs];
                for (int j = 0; j < numGlyphs; ++j) {
                    this.horizontalAdvances[j] = Float.MAX_VALUE;
                }
            }
            else {
                this.segHorizontalAdvances = new float[(numGlyphs + 32 - 1) / 32][];
            }
        }
        return;
    }
    
    @Override
    public int getNumGlyphs() {
        return this.fileFont.getNumGlyphs();
    }
    
    long getGlyphImageFromNative(int n) {
        if (FontUtilities.isWindows) {
            return this.getGlyphImageFromWindows(n);
        }
        return this.getGlyphImageFromX11(n);
    }
    
    private native long _getGlyphImageFromWindows(String p0, int p1, int p2, int p3, boolean p4, int p5);
    
    long getGlyphImageFromWindows(int i) {
        String familyName = this.fileFont.getFamilyName(null);
        int j = (this.desc.style & 0x1) | (this.desc.style & 0x2) | this.fileFont.getStyle();
        int intPtSize = this.intPtSize;
        long getGlyphImageFromWindows = this._getGlyphImageFromWindows(familyName, j, intPtSize, i, this.desc.fmHint == 2, ((TrueTypeFont)this.fileFont).fontDataSize);
        if (getGlyphImageFromWindows != 0L) {
            StrikeCache.unsafe.putFloat(getGlyphImageFromWindows + (long)StrikeCache.xAdvanceOffset, this.getGlyphAdvance(i, false));
            return getGlyphImageFromWindows;
        }
        if (FontUtilities.isLogging()) {
            FontUtilities.getLogger().warning("Failed to render glyph using GDI: code=" + i + ", fontFamily=" + familyName + ", style=" + j + ", size=" + intPtSize);
        }
        return this.fileFont.getGlyphImage(this.pScalerContext, i);
    }
    
    long getGlyphImageFromX11(int n) {
        int n2 = this.fileFont.glyphToCharMap[n];
        for (int i = 0; i < this.nativeStrikes.length; ++i) {
            CharToGlyphMapper mapper = this.fileFont.nativeFonts[i].getMapper();
            int n3 = mapper.charToGlyph((char)n2) & 0xFFFF;
            if (n3 != mapper.getMissingGlyphCode()) {
                long glyphImagePtrNoCache = this.nativeStrikes[i].getGlyphImagePtrNoCache(n3);
                if (glyphImagePtrNoCache != 0L) {
                    return glyphImagePtrNoCache;
                }
            }
        }
        return this.fileFont.getGlyphImage(this.pScalerContext, n);
    }
    
    @Override
    long getGlyphImagePtr(int i) {
        if (i >= 65534) {
            return StrikeCache.invisibleGlyphPtr;
        }
        long n;
        if ((n = this.getCachedGlyphPtr(i)) != 0L) {
            return n;
        }
        if (this.useNatives) {
            n = this.getGlyphImageFromNative(i);
            if (n == 0L && FontUtilities.isLogging()) {
                FontUtilities.getLogger().info("Strike for " + this.fileFont + " at size = " + this.intPtSize + " couldn't get native glyph for code = " + i);
            }
        }
        if (n == 0L) {
            n = this.fileFont.getGlyphImage(this.pScalerContext, i);
        }
        return this.setCachedGlyphPtr(i, n);
    }
    
    @Override
    void getGlyphImagePtrs(int[] array, long[] array2, int n) {
        for (int i = 0; i < n; ++i) {
            int n2 = array[i];
            if (n2 >= 65534) {
                array2[i] = StrikeCache.invisibleGlyphPtr;
            }
            else {
                int n3 = i;
                long cachedGlyphPtr = this.getCachedGlyphPtr(n2);
                array2[n3] = cachedGlyphPtr;
                if (cachedGlyphPtr == 0L) {
                    long n4 = 0L;
                    if (this.useNatives) {
                        n4 = this.getGlyphImageFromNative(n2);
                    }
                    if (n4 == 0L) {
                        n4 = this.fileFont.getGlyphImage(this.pScalerContext, n2);
                    }
                    array2[i] = this.setCachedGlyphPtr(n2, n4);
                }
            }
        }
    }
    
    @Override
    int getSlot0GlyphImagePtrs(int[] array, long[] array2, int n) {
        int n2 = 0;
        int i = 0;
        while (i < n) {
            int n3 = array[i];
            if (n3 >>> 24 != 0) {
                return n2;
            }
            ++n2;
            if (n3 >= 65534) {
                array2[i] = StrikeCache.invisibleGlyphPtr;
            }
            else {
                int n4 = i;
                long cachedGlyphPtr = this.getCachedGlyphPtr(n3);
                array2[n4] = cachedGlyphPtr;
                if (cachedGlyphPtr == 0L) {
                    long n5 = 0L;
                    if (this.useNatives) {
                        n5 = this.getGlyphImageFromNative(n3);
                    }
                    if (n5 == 0L) {
                        n5 = this.fileFont.getGlyphImage(this.pScalerContext, n3);
                    }
                    array2[i] = this.setCachedGlyphPtr(n3, n5);
                }
            }
            ++i;
        }
        return n2;
    }
    
    long getCachedGlyphPtr(int n) {
        try {
            return this.getCachedGlyphPtrInternal(n);
        }
        catch (Exception ex) {
            return ((NullFontScaler)FontScaler.getNullScaler()).getGlyphImage(NullFontScaler.getNullScalerContext(), n);
        }
    }
    
    private long getCachedGlyphPtrInternal(int n) {
        switch (this.glyphCacheFormat) {
            case 1: {
                return (long)this.intGlyphImages[n] & 0xFFFFFFFFL;
            }
            case 3: {
                int n2 = n >> 5;
                if (this.segIntGlyphImages[n2] != null) {
                    return (long)this.segIntGlyphImages[n2][n % 32] & 0xFFFFFFFFL;
                }
                return 0L;
            }
            case 2: {
                return this.longGlyphImages[n];
            }
            case 4: {
                int n3 = n >> 5;
                if (this.segLongGlyphImages[n3] != null) {
                    return this.segLongGlyphImages[n3][n % 32];
                }
                return 0L;
            }
            default: {
                return 0L;
            }
        }
    }
    
    private synchronized long setCachedGlyphPtr(int n, long n2) {
        try {
            return this.setCachedGlyphPtrInternal(n, n2);
        }
        catch (Exception ex) {
            switch (this.glyphCacheFormat) {
                case 1:
                case 3: {
                    StrikeCache.freeIntPointer((int)n2);
                    break;
                }
                case 2:
                case 4: {
                    StrikeCache.freeLongPointer(n2);
                    break;
                }
            }
            return ((NullFontScaler)FontScaler.getNullScaler()).getGlyphImage(NullFontScaler.getNullScalerContext(), n);
        }
    }
    
    private long setCachedGlyphPtrInternal(int n, long n2) {
        switch (this.glyphCacheFormat) {
            case 1: {
                if (this.intGlyphImages[n] == 0) {
                    this.intGlyphImages[n] = (int)n2;
                    return n2;
                }
                StrikeCache.freeIntPointer((int)n2);
                return (long)this.intGlyphImages[n] & 0xFFFFFFFFL;
            }
            case 3: {
                int n3 = n >> 5;
                int n4 = n % 32;
                if (this.segIntGlyphImages[n3] == null) {
                    this.segIntGlyphImages[n3] = new int[32];
                }
                if (this.segIntGlyphImages[n3][n4] == 0) {
                    this.segIntGlyphImages[n3][n4] = (int)n2;
                    return n2;
                }
                StrikeCache.freeIntPointer((int)n2);
                return (long)this.segIntGlyphImages[n3][n4] & 0xFFFFFFFFL;
            }
            case 2: {
                if (this.longGlyphImages[n] == 0L) {
                    return this.longGlyphImages[n] = n2;
                }
                StrikeCache.freeLongPointer(n2);
                return this.longGlyphImages[n];
            }
            case 4: {
                int n5 = n >> 5;
                int n6 = n % 32;
                if (this.segLongGlyphImages[n5] == null) {
                    this.segLongGlyphImages[n5] = new long[32];
                }
                if (this.segLongGlyphImages[n5][n6] == 0L) {
                    return this.segLongGlyphImages[n5][n6] = n2;
                }
                StrikeCache.freeLongPointer(n2);
                return this.segLongGlyphImages[n5][n6];
            }
            default: {
                this.initGlyphCache();
                return this.setCachedGlyphPtr(n, n2);
            }
        }
    }
    
    private synchronized void initGlyphCache() {
        int numGlyphs = this.mapper.getNumGlyphs();
        int glyphCacheFormat;
        if (this.segmentedCache) {
            int n = (numGlyphs + 32 - 1) / 32;
            if (FileFontStrike.longAddresses) {
                glyphCacheFormat = 4;
                this.segLongGlyphImages = new long[n][];
                this.disposer.segLongGlyphImages = this.segLongGlyphImages;
            }
            else {
                glyphCacheFormat = 3;
                this.segIntGlyphImages = new int[n][];
                this.disposer.segIntGlyphImages = this.segIntGlyphImages;
            }
        }
        else if (FileFontStrike.longAddresses) {
            glyphCacheFormat = 2;
            this.longGlyphImages = new long[numGlyphs];
            this.disposer.longGlyphImages = this.longGlyphImages;
        }
        else {
            glyphCacheFormat = 1;
            this.intGlyphImages = new int[numGlyphs];
            this.disposer.intGlyphImages = this.intGlyphImages;
        }
        this.glyphCacheFormat = glyphCacheFormat;
    }
    
    @Override
    float getGlyphAdvance(int n) {
        return this.getGlyphAdvance(n, true);
    }
    
    private float getGlyphAdvance(int n, boolean b) {
        if (n >= 65534) {
            return 0.0f;
        }
        if (this.horizontalAdvances != null) {
            float x2 = this.horizontalAdvances[n];
            if (x2 != Float.MAX_VALUE) {
                if (!b && this.invertDevTx != null) {
                    Point2D.Float float2 = new Point2D.Float(x2, 0.0f);
                    this.desc.devTx.deltaTransform(float2, float2);
                    return float2.x;
                }
                return x2;
            }
        }
        else if (this.segmentedCache && this.segHorizontalAdvances != null) {
            float[] array = this.segHorizontalAdvances[n >> 5];
            if (array != null) {
                float x = array[n % 32];
                if (x != Float.MAX_VALUE) {
                    if (!b && this.invertDevTx != null) {
                        Point2D.Float float1 = new Point2D.Float(x, 0.0f);
                        this.desc.devTx.deltaTransform(float1, float1);
                        return float1.x;
                    }
                    return x;
                }
            }
        }
        if (!b && this.invertDevTx != null) {
            Point2D.Float o = new Point2D.Float();
            this.fileFont.getGlyphMetrics(this.pScalerContext, n, o);
            return o.x;
        }
        float n3;
        if (this.invertDevTx != null || !b) {
            n3 = this.getGlyphMetrics(n, b).x;
        }
        else {
            long n2;
            if (this.getImageWithAdvance) {
                n2 = this.getGlyphImagePtr(n);
            }
            else {
                n2 = this.getCachedGlyphPtr(n);
            }
            if (n2 != 0L) {
                n3 = StrikeCache.unsafe.getFloat(n2 + (long)StrikeCache.xAdvanceOffset);
            }
            else {
                n3 = this.fileFont.getGlyphAdvance(this.pScalerContext, n);
            }
        }
        if (this.horizontalAdvances != null) {
            this.horizontalAdvances[n] = n3;
        }
        else if (this.segmentedCache && this.segHorizontalAdvances != null) {
            int n4 = n >> 5;
            int n5 = n % 32;
            if (this.segHorizontalAdvances[n4] == null) {
                this.segHorizontalAdvances[n4] = new float[32];
                for (int i = 0; i < 32; ++i) {
                    this.segHorizontalAdvances[n4][i] = Float.MAX_VALUE;
                }
            }
            this.segHorizontalAdvances[n4][n5] = n3;
        }
        return n3;
    }
    
    @Override
    float getCodePointAdvance(int n) {
        return this.getGlyphAdvance(this.mapper.charToGlyph(n));
    }
    
    @Override
    void getGlyphImageBounds(int n, Point2D.Float float1, Rectangle rectangle) {
        long glyphImagePtr = this.getGlyphImagePtr(n);
        if (glyphImagePtr == 0L) {
            rectangle.x = (int)Math.floor((double)float1.x);
            rectangle.y = (int)Math.floor((double)float1.y);
            int n2 = 0;
            rectangle.height = n2;
            rectangle.width = n2;
            return;
        }
        float float2 = StrikeCache.unsafe.getFloat(glyphImagePtr + (long)StrikeCache.topLeftXOffset);
        float float3 = StrikeCache.unsafe.getFloat(glyphImagePtr + (long)StrikeCache.topLeftYOffset);
        rectangle.x = (int)Math.floor((double)(float1.x + float2));
        rectangle.y = (int)Math.floor((double)(float1.y + float3));
        rectangle.width = (StrikeCache.unsafe.getShort(glyphImagePtr + (long)StrikeCache.widthOffset) & 0xFFFF);
        rectangle.height = (StrikeCache.unsafe.getShort(glyphImagePtr + (long)StrikeCache.heightOffset) & 0xFFFF);
        if ((this.desc.aaHint == 4 || this.desc.aaHint == 5) && float2 <= -2.0f && this.getGlyphImageMinX(glyphImagePtr, rectangle.x) > rectangle.x) {
            ++rectangle.x;
            --rectangle.width;
        }
        return;
    }
    
    private int getGlyphImageMinX(long n, int n2) {
        int char1 = StrikeCache.unsafe.getChar(n + (long)StrikeCache.widthOffset);
        int char2 = StrikeCache.unsafe.getChar(n + (long)StrikeCache.heightOffset);
        int char3 = StrikeCache.unsafe.getChar(n + (long)StrikeCache.rowBytesOffset);
        if (char3 == char1) {
            return n2;
        }
        long address = StrikeCache.unsafe.getAddress(n + (long)StrikeCache.pixelDataOffset);
        if (address == 0L) {
            return n2;
        }
        for (int i = '\0'; i < char2; ++i) {
            int j = 0;
            while (j < 3) {
                if (StrikeCache.unsafe.getByte(address + (long)(i * char3) + (long)j) != 0) {
                    return n2;
                }
                ++j;
            }
        }
        return n2 + 1;
    }
    
    @Override
    StrikeMetrics getFontMetrics() {
        if (this.strikeMetrics == null) {
            this.strikeMetrics = this.fileFont.getFontMetrics(this.pScalerContext);
            if (this.invertDevTx != null) {
                this.strikeMetrics.convertToUserSpace(this.invertDevTx);
            }
        }
        return this.strikeMetrics;
    }
    
    @Override
    Point2D.Float getGlyphMetrics(int n) {
        return this.getGlyphMetrics(n, true);
    }
    
    private Point2D.Float getGlyphMetrics(int i, boolean b) {
        Point2D.Float float1 = new Point2D.Float();
        if (i >= 65534) {
            return float1;
        }
        long n;
        if (this.getImageWithAdvance && b) {
            n = this.getGlyphImagePtr(i);
        }
        else {
            n = this.getCachedGlyphPtr(i);
        }
        if (n != 0L) {
            float1 = new Point2D.Float();
            float1.x = StrikeCache.unsafe.getFloat(n + (long)StrikeCache.xAdvanceOffset);
            float1.y = StrikeCache.unsafe.getFloat(n + (long)StrikeCache.yAdvanceOffset);
            if (this.invertDevTx != null) {
                this.invertDevTx.deltaTransform(float1, float1);
            }
        }
        else {
            Integer value = Integer.valueOf(i);
            Point2D.Float float2 = null;
            Object referent = null;
            if (this.glyphMetricsMapRef != null) {
                referent = (ConcurrentHashMap<Integer, Point2D.Float>)this.glyphMetricsMapRef.get();
            }
            if (referent != null) {
                float2 = (Point2D.Float)referent.get(value);
                if (float2 != null) {
                    float1.x = float2.x;
                    float1.y = float2.y;
                    return float1;
                }
            }
            if (float2 == null) {
                this.fileFont.getGlyphMetrics(this.pScalerContext, i, float1);
                if (this.invertDevTx != null) {
                    this.invertDevTx.deltaTransform(float1, float1);
                }
                Point2D.Float value2 = new Point2D.Float(float1.x, float1.y);
                if (referent == null) {
                    referent = new ConcurrentHashMap();
                    this.glyphMetricsMapRef = new SoftReference(referent);
                }
                referent.put(value, value2);
            }
        }
        return float1;
    }
    
    @Override
    Point2D.Float getCharMetrics(char c) {
        return this.getGlyphMetrics(this.mapper.charToGlyph(c));
    }
    
    @Override
    Rectangle2D.Float getGlyphOutlineBounds(int i) {
        if (this.boundsMap == null) {
            this.boundsMap = new ConcurrentHashMap();
        }
        Integer value = Integer.valueOf(i);
        Rectangle2D.Float glyphOutlineBounds = (Rectangle2D.Float)this.boundsMap.get(value);
        if (glyphOutlineBounds == null) {
            glyphOutlineBounds = this.fileFont.getGlyphOutlineBounds(this.pScalerContext, i);
            this.boundsMap.put(value, glyphOutlineBounds);
            return glyphOutlineBounds;
        }
        return glyphOutlineBounds;
    }
    
    public Rectangle2D getOutlineBounds(int n) {
        return this.fileFont.getGlyphOutlineBounds(this.pScalerContext, n);
    }
    
    @Override
    GeneralPath getGlyphOutline(int n, float n2, float n3) {
        GeneralPath glyphOutline = null;
        Object referent = null;
        if (this.outlineMapRef != null) {
            referent = (ConcurrentHashMap)this.outlineMapRef.get();
            if (referent != null) {
                glyphOutline = (GeneralPath)referent.get(Integer.valueOf(n));
            }
        }
        if (glyphOutline == null) {
            glyphOutline = this.fileFont.getGlyphOutline(this.pScalerContext, n, 0.0f, 0.0f);
            if (referent == null) {
                referent = new ConcurrentHashMap();
                this.outlineMapRef = new WeakReference(referent);
            }
            referent.put(Integer.valueOf(n), glyphOutline);
        }
        GeneralPath generalPath = (GeneralPath)glyphOutline.clone();
        if (n2 != 0.0f || n3 != 0.0f) {
            generalPath.transform(AffineTransform.getTranslateInstance((double)n2, (double)n3));
            return generalPath;
        }
        return generalPath;
    }
    
    @Override
    GeneralPath getGlyphVectorOutline(int[] array, float n, float n2) {
        return this.fileFont.getGlyphVectorOutline(this.pScalerContext, array, array.length, n, n2);
    }
    
    @Override
    protected void adjustPoint(Point2D.Float float1) {
        if (this.invertDevTx != null) {
            this.invertDevTx.deltaTransform(float1, float1);
        }
    }
    
    static {
        FileFontStrike.isXPorLater = false;
        if (FontUtilities.isWindows && !FontUtilities.useT2K && !GraphicsEnvironment.isHeadless()) {
            FileFontStrike.isXPorLater = initNative();
        }
    }
}
